/*
 version using the SPIFFS file system to store button codes
 version handling the learning remote function
 data is transmitted using the RMT peripheral block
 data coming in from remote UDP source is just an ASCII number representing button number
 UDP local port address is 40000
 local IP # for my ESP32 is 192.168.2.17
 */

#include "esp32_rmt.h"
#include "FS.h"
#include "SD.h"
#include "SPI.h"
#include "FS.h"
#include "SPIFFS.h"

// following are libraries for the Wi-Fi functions
#include <WiFiUdp.h>
#include <WiFi.h>


#define loadSwitch 4   // I04 used for load switch- pressing this during reset will activate the EEPROM load from SD card
#define TSMP1138Data 2  // I02 is used, along with the RMT receive input pin, to sense when data is coming in from IR remote

/* LEDC peripheral used only in early testing to provide a waveform to feed into RMT for testing*/
/*
#define LEDC_TIMER_13_BIT  13
#define LEDC_BASE_FREQ     2000
#define LEDC_CHANNEL_0     0
#define LED_PIN1            15
*/


char ssid[] = "xxxxxxxx";  //  your network SSID (name)
char pass[] = "xxxxxxxx";       // your network password
 
								 
WiFiUDP Udp; // instantiate a UDP object to let us receive UDP packets from iPad

#define eventCount 300   // maximum number of pulse/space combinations for each button code 
const int maxButtons=75;      // defines the maximum number of buttons that can be stored

 // instantiate ESP32 Remote and WifiUDP classes
ESP32_RMT rem1,rem2;
WiFiUDP udp;

unsigned int localPort = 40000;      // local port to listen for UDP packets
uint8_t cnt;
uint16_t cmd, addr;
int pointer;
int cb;
int CarrierFrequency;
int space[eventCount];
int pulse[eventCount];
int IRPointer;
bool overflowFlag = false;
unsigned int LastButtonNumber=0;
int ButtonNumber=0;
char packetBuffer[10]; //buffer to hold incoming  packets
bool learnModeFlag = false;
bool learnUDPFlag = false;
uint8_t buf[3000];
uint16_t BufSize;
uint8_t * ptr;


/*  SPIFFS file system routines follow*/
/* These must be placed at head of code or they won't be recognized */

void listDir(fs::FS &fs, const char * dirname, uint8_t levels) {
	Serial.printf("Listing directory: %s\n", dirname);
	File root = fs.open(dirname);
	if (!root) {
		Serial.println("Failed to open directory");
		return;
	}
	if (!root.isDirectory()) {
		Serial.println("Not a directory");
		return;
	}

	File file = root.openNextFile();
	while (file) {
		if (file.isDirectory()) {
			Serial.print("  DIR : ");
			Serial.println(file.name());
			if (levels) {
				listDir(fs, file.name(), levels - 1);
			}
		}
		else {
			Serial.print("  FILE: ");
			Serial.print(file.name());
			Serial.print("  SIZE: ");
			Serial.println(file.size());
		}
		file = root.openNextFile();
	}
}

void readFile(fs::FS &fs, const char * path) {
	Serial.printf("Reading file: %s\n", path);

	File file = fs.open(path);
	if (!file || file.isDirectory()) {
		Serial.println("Failed to open file for reading");
		return;
	}
	int i = 0;
   while(file.available()){
       buf[i++]=file.read();
	   }
    BufSize = i;
}
void writeFile(fs::FS &fs,  char * path, int BufSize) {   // was const char *
	Serial.printf("Writing file: %s\n", path);

	File file = fs.open(path, FILE_WRITE);
	if (!file) {
		Serial.println("Failed to open file for writing");
		return;
	}

	if (file.write(buf, BufSize)) {
		Serial.print(BufSize);
		Serial.println(" bytes written");
	}
	else {
		Serial.println("Write byte failed");
	}
}

void setup() {
// set up the LEDC to produce a square wave pulse to test the IR receiver function in place of actual IR receiver
// only used early on in testsing
/*
ledcSetup(LEDC_CHANNEL_0, LEDC_BASE_FREQ, LEDC_TIMER_13_BIT);
	ledcAttachPin(LED_PIN1, LEDC_CHANNEL_0);
	ledcWrite(LEDC_CHANNEL_0, 4096);
*/

	pinMode(loadSwitch, INPUT_PULLUP);
	pinMode(TSMP1138Data, INPUT);

	if (digitalRead(loadSwitch) == 0)
	{
		learnModeFlag = true;
	}
	Serial.begin(115200);
	delay(1000);
	
	if (!SPIFFS.begin(true)) {    // very important to use SPIFFS.begin(true) not SPIFFS.begin() or mount will fail
		Serial.println("SPIFFS Mount Failed");
		while (1);;;   // hang here
	}
	Serial.println("SPIFFS mounted");
	memset(buf, 0, sizeof(buf));  // zero out the storage array
	rem1.begin(16, 1);   // initialize an instance of the IR remote class (RMT) TRANSMIT using pin 16 as output pin, and blocking -"WAIT TIL SENT"
	rem2.begin(17, 0);   // initialize an instance of IR remote class (RMT) receive using pin 17 as input pin
	rem2.rmt_rx_init();   //  set up the RMT receive channel -esp32_rmt.cpp is coded to use only 1 memory block (0) since RMT receiver is only used to capture the Carrier frequency accurately
		
	// start wifi and UDP
	WiFi.begin(ssid, pass);
	while (WiFi.status() != WL_CONNECTED) {
		delay(500);
		Serial.print(".");
	}
	Serial.println("");
	Serial.println("WiFi connected");
	// Print the IP address
	Serial.print("IP# = ");
	Serial.println(WiFi.localIP());
	Serial.print("MAC address= ");
	String MAC;
	MAC = WiFi.macAddress();
	Serial.println(MAC);
	udp.begin(localPort);  // start UDP client on localPort

	if (learnModeFlag)
	{
		learnMode();
	}
	Serial.println("waiting for UDP button numbers");
}

// this is the normal operational mode loop
void loop() {
	cb = udp.parsePacket();
	if (cb > 0) {
		for (int i = 0; i < sizeof(packetBuffer); i++) {
			packetBuffer[i] = 0;   // clear packet buffer
		}
		//		Serial.print("packet received, length=");
		//		Serial.println(cb);
		// We've received a packet, read the data from it
		udp.read(packetBuffer, cb); // read the packet into the buffer
		packetBuffer[cb] = 0; // place an ASCIIZ terminator
		int command = atoi(packetBuffer);
		if (command > 0 && command < maxButtons + 1) {
			ButtonNumber = command;
			Serial.print("button # ");
			Serial.println(ButtonNumber);
			if (ButtonNumber != LastButtonNumber) {
				// form a filename-  /buttonNumber.but
				String fn;
				char  f[20];
				fn = "/";
				fn = fn + String(ButtonNumber);
				fn = fn + ".but";
				int l = fn.length();
				fn.toCharArray(f, l + 1);
				memset(buf, 0, sizeof(buf));  // zero out the storage array
				readFile(SPIFFS, f);          // load button code into buf[] array  from ESP32 SPI filesystem 
				retrieveCarrierFrequency();
			}
			if (CarrierFrequency > 20000) {
				rem1.rmt_tx_init(CarrierFrequency);
				ptr = buf;
				ptr += 2;  // skip the 2 Carrier frequency bytes
				rem1.IRSendRaw(ptr);   // actually send out the IR code via LED
				LastButtonNumber = ButtonNumber;
			}
		}
		else {
			if (command > maxButtons) {
				Serial.print("Button number greater than maximum Button number of ");
				Serial.println(maxButtons);
			}
		}
	}
}
void getButtonNumber() {
	String a;
	int b;
	Serial.print("Current button number = ");
	Serial.print(ButtonNumber);
	Serial.print ( " Enter button # (or Enter or next in line)");
	Serial.setTimeout(600000); // 600 secs timeout
	a = Serial.readStringUntil('\r');
	b = a.length();
	if (b ==0 ) {     // only a CR pressed or timeout
		ButtonNumber++;
	}
	else
	   {
		ButtonNumber = a.toInt();
		Serial.println(ButtonNumber);
	   }
	}

int saveToBuffer() {
	uint16_t MSB, LSB, addrSave;
	addr = 0;
//	addrSave = addr;
	Serial.println("SPIFILE save");
	MSB = CarrierFrequency >> 8;
	LSB = CarrierFrequency & 0xff;
	buf[addr++] = MSB;
	buf[addr++] = LSB;
	for (int i = 0; i < IRPointer; i++) {
		unsigned int p, s;
		p = pulse[i];
		while (1) {
			if (p < 32510) {   // handle pulse/spaces less than 32510 us
				MSB = 0x80 + (p >> 8);
				LSB = p & 0xff;
				buf[addr++] = MSB;
				buf[addr++] = LSB;
				break;
			}
			else {            // break up a pulse that is longer than 32510 us. into multiple pulses which are < 32510 (0x7eFe+ MSB set) (RMT allows up to 32767)
				MSB = 0xfe;   // RMT allows up to 32767 (+ MSB set for pulses) but avoid using FF FF as that is value of unwritten EEPROM cells
				LSB = 0xfe;   // ""
				buf[addr++] = MSB;
				buf[addr++] = LSB;
				p -= 32510;
			}
		}
		s = space[i];
		if (i < IRPointer - 1) {    // skip the final space value as it is unneeded (=time until 2 sec timeout)
			while (1) {
				if (s < 32510) {
					MSB = s >> 8;
					LSB = s & 0xff;
					buf[addr++] = MSB;
					buf[addr++] = LSB;
					break;
				}
				else {            // break up a space that is longer than 32510 us. into multiple spaces which are less than 32510 (RMT allows up to 32767)
					MSB = 0x7e;   // avoid using FF FF as that is value of unwritten EEPROM cells
					LSB = 0xfe;   // ""
					buf[addr++] = MSB;
					buf[addr++] = LSB;
					s -= 32510;
				}
			}
		}
	}
	buf[addr++]=0;
	buf[addr++]= 0;         // terminating value = 2 zero integers
	buf[addr++]= 0;
	buf[addr++]=0;
	Serial.print("pulse/space events= ");
	Serial.println((addr)>>1);
	return addr;
}

void retrieveCarrierFrequency() {
	uint16_t MSB, LSB;
	MSB = buf[0];
	LSB = buf[1];
	CarrierFrequency = (MSB << 8) + LSB;
//	Serial.print("Carrier Freq. = ");
//	Serial.println(CarrierFrequency);
}


void disableReceiverInterrupts() {
	int * ptr3;
	ptr3 = (int *)RMT_INT_ENA_REG;
	int a = *ptr3;
	a = a & 0xFFFFFFFC;  // clear the  the channel 0 Rx and Tx interrupt enable bits
	*ptr3 = a;
}

   /* This routine uses the micros() function and digitalRead command to capture the IR pulses coming from the TSMP1138 "Learning" IR receiver */
   /*  As well as using the RMT peripheral's Receive function to get an accurate reading of carrier frequency */
   
void analyseIR() {
	unsigned long  d, e, f, g, j, pulseStart, timeout;
	bool timeoutFlag = false;
	overflowFlag = false;
	int ncycles = 0;
	bool start = true;
	memset(pulse, 0, sizeof(pulse));  // zero out the storage arrays
	memset(space, 0, sizeof(space));
	IRPointer = 0;
	// Start up the RMT receiver block- this block is only used to determina an accurate carrier frequency
	rmt_rx_start((rmt_channel_t) 0, 1);  // start RMT receiver and reset memory pointer to 0
	disableReceiverInterrupts();         // needed or ESP32 crashes- probably because RMT interrupt handler not defined  
	Serial.println("waiting");
	// accuracy of this routine, when measured using a stable clock signal, was +0  -> -3 us
	while (digitalRead(TSMP1138Data) == 1) {}    // 1st falling edge
	e = micros();    // time of leading falling edge
	pulseStart = e;
	timeout = e + 1000000; // 1 seconds timeout
	while (1) {
		while (digitalRead(TSMP1138Data) == 0) {}   //wait til first rising edge
		while (digitalRead(TSMP1138Data) == 1) {    // and subsequent falling edge
			d = micros();
			if (d > timeout) {
				timeoutFlag = true;
				break;
			}
		}
		f = micros();
		g = f - e;  // get time of 1 complete cycle down/up/down
		if (g < 100) {
			ncycles++;
			if (ncycles == 4 && start == true) {   // after 4 cycles, store 1/2 of the carrier period in us (to adjust the pulse and space intervals)
				j = g >> 1;
				start = false;
			}
		}
		else {
			space[IRPointer] = g - j;  // subtract out 1/2 carrier cycle time
			pulse[IRPointer] = f - (pulseStart + g - j); // add in 1/2 carrier cycle time 
			pulseStart = f;
			IRPointer++;
			start = false;
			ncycles = 0;
			if (IRPointer > eventCount) {
				overflowFlag = true;
				break;
			}
		}
		e = micros();
		if (timeoutFlag == true) break;
	}
	if (overflowFlag == true)   Serial.println("Code too long");
	Serial.println("1 sec timeout elapsed");
// now determina an accurate value for the carrier frequency:
// examine the RMT receiver buffer and isolate cycles 4-12 of the input IR stream
// this will be a valid part of the leading "sync" pulse sent by all IR remotes to allow the IR receiver to set its AGC properly
// find total period over the 4 cycles, div by 4 and reciprocate

	char *ptr2;
	byte msb1, lsb1, msb2, lsb2;
	unsigned int a, b;
	ptr2 = (char *)RMT_CHANNEL_MEM(0);  
	ptr2 += 8;   //skip 1st 4 carrier cycles
	int c = 0;
	for (int i = 0; i < 8; i++) {     // read 8 full cycles of carrier
		lsb1 = *ptr2++;
		msb1 = *ptr2++;
		lsb2 = *ptr2++;
		msb2 = *ptr2++;
		a = lsb1 + (msb1 << 8);
		if (a > 32767) {
			c = c + (a - 32768);
		}
		else {
			c = c + a;
		}
		b = lsb2 + (msb2 << 8);
		if (b > 32767) {
			c = c + (b - 32768);
		}
		else {
			c = c + b;
		}
	}
	CarrierFrequency=8000000/c;   // c is = 8 full periods of carrier wave, so freq = 8 million/c
	Serial.print("carrier freq ");
	Serial.println(CarrierFrequency);
	}

void learnMode() {
	while (1) {
		if (learnUDPFlag == true) {
			getButtonNumberViaUDP();
		}
		else {
			getButtonNumber();
		}
		if (ButtonNumber == 0) learnUDPFlag = true;   // if you enter a button number of zero, it enables the UDP routine to present the button number
		if (ButtonNumber > 0 && ButtonNumber < maxButtons + 1) {
			analyseIR();
			BufSize= saveToBuffer();  // translate the pulse/space array into a form directly useable by RMT, and save in RAM array buf[]
			for (int i = 0; i < BufSize; i++) {
				Serial.println(buf[i]);
			}
			Serial.println("************************************************");
			String fn;
			char  f[20];
			fn = "/";
			fn = fn+ String(ButtonNumber);
			fn = fn + ".but";
			int l = fn.length();
			fn.toCharArray(f, l + 1);
			writeFile(SPIFFS, f,BufSize);
			memset(buf, 0, sizeof(buf));  // zero out the storage array
			readFile(SPIFFS, f);
			for (int i = 0; i < BufSize; i++) {
				Serial.println(buf[i]);
			}
		}
	}
}

void getButtonNumberViaUDP() {
	Serial.println("Waiting for button number via UDP");
	while (1) {
		cb = udp.parsePacket();
		if (cb > 0) {
			for (int i = 0; i < sizeof(packetBuffer); i++) {
				packetBuffer[i] = 0;   // clear packet buffer
			}
			// We've received a packet, read the data from it
			udp.read(packetBuffer, cb); // read the packet into the buffer
			packetBuffer[cb] = 0; // place an ASCIIZ terminator
			int command = atoi(packetBuffer);
			if (command > 0 && command < maxButtons + 1) {
				ButtonNumber = command;
				Serial.print("Received button # ");
				Serial.println(ButtonNumber);
				break;
			}

		}
	}
}
