#include <stdio.h>
#include <string.h>
#include "esp32_rmt.h"

#define TRANSMIT 1
#define RECEIVE 0
// following added by BM


//#define RMT_CLK_DIV 100 /*!< RMT counter clock divider */   // defined in rmt.h


#if RMT_RX_SELF_TEST
#define RMT_RX_ACTIVE_LEVEL  0   /*!< Data bit is active high for self test mode */
#define RMT_TX_CARRIER_EN    0   /*!< Disable carrier for self test mode  */
#else
//Test with infrared LED, we have to enable carrier for transmitter
//When testing via IR led, the receiver waveform is usually active-low.
#define RMT_RX_ACTIVE_LEVEL  0   /*!< If we connect with a IR receiver, the data is active low */
#define RMT_TX_CARRIER_EN    1   /*!< Enable carrier for IR transmitter test with IR led */
#endif


ESP32_RMT::ESP32_RMT()
{
	//printf("booting remote peripheral\n\r");
	//printf("hello world\n\r");
}

void ESP32_RMT::begin(uint8_t pin, bool mode)
{
	if(mode==TRANSMIT)
	{

		txPin = pin;
		printf("IR Tx pin: %d \n\r", txPin);
	}
	if (mode == RECEIVE)
	{
		rxPin = pin;
		printf("IR Rx pin:%d \n\r", rxPin);

	}
}


void ESP32_RMT::rmt_tx_init(int freq)   // this code added by BM
{
	rmt_config_t rmt_tx;
	rmt_tx.channel = (rmt_channel_t)txChannel;
	rmt_tx.gpio_num = (gpio_num_t)txPin;
	rmt_tx.mem_block_num = 6;  
	rmt_tx.clk_div = RMT_CLK_DIV;
	rmt_tx.tx_config.loop_en = false;
	rmt_tx.tx_config.carrier_duty_percent = 50;
	rmt_tx.tx_config.carrier_freq_hz = freq;
	rmt_tx.tx_config.carrier_level = (rmt_carrier_level_t)1;
	rmt_tx.tx_config.carrier_en = 1;  // RMT_TX_CARRIER_EN;
	rmt_tx.tx_config.idle_level = (rmt_idle_level_t)0;
	rmt_tx.tx_config.idle_output_en = true;
	rmt_tx.rmt_mode = (rmt_mode_t)0;
	rmt_config(&rmt_tx);
	rmt_driver_install(rmt_tx.channel, 0, RMT_INTR_NUM);
}

void ESP32_RMT::rmt_rx_init() {			// this code added by BM

	rmt_config_t rmt_rx;
	rmt_rx.channel = (rmt_channel_t)rxChannel;    
	rmt_rx.gpio_num = (gpio_num_t)rxPin;
	rmt_rx.clk_div =  RMT_CLK_DIV;
	rmt_rx.mem_block_num = 1;   
	rmt_rx.rmt_mode = (rmt_mode_t)RMT_MODE_RX;
	rmt_rx.rx_config.filter_en = true;
	rmt_rx.rx_config.filter_ticks_thresh = (uint8_t) 20; 
	rmt_rx.rx_config.idle_threshold = rmt_item32_tIMEOUT_US / 10 * (RMT_TICK_10_US);
	rmt_config(&rmt_rx);
	rmt_driver_install(rmt_rx.channel, 1000, 0);
	
}


