#include <stdint.h>
#include "driver/rmt.h"


#ifndef ESP32_RMT_H
#define ESP32_RMT_H


#define RMT_TX_CHANNEL    4     /*!< RMT channel for transmitter */
#define RMT_RX_CHANNEL    0     /*!< RMT channel for receiver */
//#define RMT_RX_GPIO_NUM  19     /*!< GPIO number for receiver */
#define RMT_INTR_NUM     19     /*!< RMT interrupt number, select from soc.h */
#define RMT_CLK_DIV      80    /*!< RMT counter clock divider  set for 1 uS per tick*/        
#define RMT_TICK_10_US    (80000000/RMT_CLK_DIV/100000)   /*!< RMT counter  */    // don't use this constant now because it is actaully 3.333, which =3 (10% error)

#define RMT_TX_DATA_NUM  1    /*!<  tx test data number */
#define rmt_item32_tIMEOUT_US  98000   /*!< RMT receiver timeout value(us) = 98 mS*/

class ESP32_RMT
{
	public:
		ESP32_RMT(); 
		void begin(uint8_t, bool);	

		//IR transmit/receive  functions	
		//  added to class by BM
		void IRSend(int, int, int, int, int, unsigned int, int, int, int,unsigned int, int, int,int,unsigned int, int, int, 
			int, int, int , int,int,int, int);
		void IRSendRaw(uint8_t *);
		void rmt_rx_init();        
		void rmt_tx_init(int);   // this was empty (in esp32_rmt.cpp) but has BM code now
//		void necSend(uint16_t, uint16_t);   // only need this for Necsend example
	private: 

		// following functions added to class by BM
		
		void IR_fill_item_level(rmt_item32_t*, int, int);
		void IR_fill_item_header(rmt_item32_t*,int phead,int shead);
		void IR_fill_item_bit_one(rmt_item32_t*,int,int);
		void IR_fill_item_bit_zero(rmt_item32_t*,int,int);
		void IR_fill_item_gap(rmt_item32_t*, int);
		void IR_fill_item_pulse(rmt_item32_t*, int);
		void IR_fill_item_doublePulse(rmt_item32_t* item, int, int);
		void IR_fill_item_doubleSpace(rmt_item32_t* item, int, int);
/*      omly need this to use necsend example  
		void nec_fill_item_level(rmt_item32_t*, int, int);
		void nec_fill_item_header(rmt_item32_t*);
		void nec_fill_item_bit_one(rmt_item32_t*);
		void nec_fill_item_bit_zero(rmt_item32_t*);
		void nec_fill_item_end(rmt_item32_t*);
		int nec_build_items(int, rmt_item32_t*, int, uint16_t, uint16_t);

*/
		uint8_t txChannel = 2;    // was 1 - now chan 4 to make more room for rec buffer
		uint8_t rxChannel = 0;
		
		uint8_t txPin =  17; //default transmit pin
		uint8_t rxPin = 19; //default receive pin
		uint16_t phead;
		uint16_t shead;
	
};

 #endif // ESP32_RMT





